import React, { useEffect } from "react";
import {
	Button,
	Dialog,
	DialogActions,
	DialogContent,
	DialogTitle,
	TextField,
} from "@mui/material";
import { useForm, Controller } from "react-hook-form";

interface AddEditCategoryModalProps {
	open: boolean;
	handleClose: () => void;
	handleUpdate: (id: string, name: string, role: string) => void;
	editRow: any;
}

interface FormData {
	category: string;
	role: string;
	name: string;
}

const AddEditCategoryModal: React.FC<AddEditCategoryModalProps> = ({
	open,
	handleClose,
	editRow,
	handleUpdate,
}) => {
	const {
		control,
		handleSubmit,
		formState: { errors },
		setValue,
		reset,
	} = useForm<FormData>({
		defaultValues: {
			category: "",
			name: "",
			role: "",
		},
	});

	const onSubmit = (data: FormData) => {
		if (editRow) {
			handleUpdate(editRow.id, data.name, data.role);
		}
		handleClose();
		reset();
	};

	useEffect(() => {
		if (editRow) {
			setValue("category", editRow.name);
			setValue("name", editRow.name);
			setValue("role", editRow.role);
		} else {
			reset();
		}
	}, [editRow, setValue, reset]);

	return (
		<>
			<Dialog
				open={open}
				onClose={handleClose}
				maxWidth="md"
				fullWidth
				PaperProps={{
					component: "form",
					onSubmit: handleSubmit(onSubmit),
				}}
			>
				<DialogTitle>{editRow ? "Edit" : "Add"} Category</DialogTitle>
				<DialogContent>
					<Controller
						name="category"
						control={control}
						rules={{
							required: "Category Name is required",
						}}
						render={({ field }) => (
							<TextField
								{...field}
								autoFocus
								required
								margin="dense"
								label="Add Category Name"
								type="text"
								fullWidth
								variant="standard"
								error={!!errors.category}
								helperText={errors.category?.message}
							/>
						)}
					/>
					<Controller
						name="name"
						control={control}
						rules={{
							required: "Admin Name is required",
						}}
						render={({ field }) => (
							<TextField
								{...field}
								autoFocus
								required
								margin="dense"
								label="Admin Name"
								type="text"
								fullWidth
								variant="standard"
								error={!!errors.name}
								helperText={errors.name?.message}
							/>
						)}
					/>

					<Controller
						name="role"
						control={control}
						rules={{
							required: "Role is required",
						}}
						render={({ field }) => (
							<TextField
								{...field}
								margin="dense"
								label="Role"
								type="text"
								fullWidth
								variant="standard"
								error={!!errors.role}
								helperText={errors.role?.message}
							/>
						)}
					/>
				</DialogContent>
				<DialogActions>
					<Button onClick={handleClose}>Cancel</Button>
					<Button type="submit">Save</Button>
				</DialogActions>
			</Dialog>
		</>
	);
};

export default AddEditCategoryModal;
